# 2019-Feb-25. Created by Maxim T. 
$scriptPath = Split-Path -parent $MyInvocation.MyCommand.Definition; 
. ($scriptPath + "\psFunctions.ps1");     # Adding script with reusable functions
. ($scriptPath + "\psSetVariables.ps1");  # Adding script to assign values to common variables

$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); 
(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: ExchRate YahooID".PadRight(28);
Get-ChildItem -Path $currExchIDFolder -Filter *Intraday.txt -File | foreach { $_.Delete()}; # Delete existing all files from intraday folder
$currExchIDFile = $currExchIDFolder + "YahooCurrExchIntraday.txt"; #if (Test-Path $currExchIDFile) { Remove-Item $currExchIDFile;} # Removing intraday file before each load

$listStart = $config.IndexOf("<Currency>"); $listEnd = $config.IndexOf("</Currency>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {"<currency> Symbol list is empty. Exiting script." | Out-File $logFile -Encoding OEM -Append; exit(1);}
$currList = @($config | Select-Object -Index(($listStart+1)..($listEnd-1))); #list of symbols we will work on
$currCount= $currList.Count; "Currency count: $currCount. MinDate: $minDate" | Out-File $logFile -Encoding OEM -Append;
if ($currCount -le 1) {"Found just one currency, will not create currency exchange database"; exit(1);}

#$urlBase = "https://finance.google.com/finance?q=@@CURRFROM@@@@CURRTO@@"; # Example: https://finance.google.com/finance?q=USDEUR
$urlBase = "https://finance.yahoo.com/quote/@@CURRFROM@@@@CURRTO@@=X"; # Example: https://finance.yahoo.com/quote/CADUSD=X

For($currFromIndex=0; $currFromIndex -lt $currCount; $currFromIndex++) {
    For($currToIndex=0; $currToIndex -lt $currCount; $currToIndex++) {
        if ($currList[$currFromIndex] -ne $currList[$currToIndex]) {
            $currFrom=$currList[$currFromIndex]; $currTo = $currList[$currToIndex]; $url = $urlBase.Replace("@@CURRFROM@@",$currFrom).Replace("@@CURRTO@@",$currTo);
            "  Requesting url: " + $url | Out-File $logFile -Encoding OEM -Append;
            $wc = new-object system.net.WebClient;
            try {$webpage = $wc.DownloadData($url); } # Get page from url. This page will contain all quotes for single Symbol
            catch { $reqFailed++; "  " + $currFrom-$currTo + " - Not Found (web err) `r`n" | Out-File $logFile -Encoding OEM -Append; continue;} # if attempt to get webpage failed go to next symbol
            $str = [System.Text.Encoding]::ASCII.GetString($webpage); # This variable now contains downloaded quotes
            $price = GetTxtBetween $str "D(ib)`" data-reactid=`"32`">" "</span>"; # Getting price/exchange value

            if ($price  -eq "" -or $price  -eq "N/A" -or $price -eq "0" -or $price -eq "0.00") {$reqFailed++; "Something wrong with Price $price" | Out-File $logFile -Encoding OEM -append; continue;} # wrong last price, ignore line
            $date = (Get-Date).ToString("yyyy-MM-dd"); $dayOfWeek = (Get-Date).DayOfWeek.ToString();
            if ($dayOfWeek -eq "Saturday" -or $dayOfWeek -eq "Sunday") {$weekendFlag = $true} else {$weekendFlag=$false};

            $ret = GetCurrExchInfo $currFrom $currTo $currExchFolder $minDate; $currExchFile = $ret[0]; $nextDate = $ret[1]; $lastQuote = $ret[2];
            "CurrExch. From: $currFrom -> To: $currTo. Next date: $nextDate. Quote file: $currExchFile" | Out-File $logFile -Encoding OEM -Append;
    
            if ($date -ge $nextDate -and (!$weekendFlag)) {
                "$date,$price,$currFrom,$currTo" | Out-File $currExchIDFile -Encoding OEM -append; $reqRowsT++; if("$date $time" -gt $lastDateTime) {$lastDateTime = "$date $time"};
                " New quote date: " + $date + " " + $time | Out-File $logFile -Encoding OEM -append; $reqSucceed++;
            }
            else {" Newer quote not found" | Out-File $logFile -Encoding OEM -append; $reqFailed++;}
        }
    }
}
$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";

(Get-Date).ToString("HH:mm:ss") + " --- Finished. CurrExch Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed. Duration: $duration. Date-Time: " + $lastDateTime + "`r`n" | Out-File $logFile -Encoding OEM -append;
$logSummary + ". CurrExch Requested/Succeed/Failed: $reqCount/$reqSucceed/$reqFailed. Date: " + $lastDateTime + ". Duration: $duration";
