# 2017-Sep-10. Created by Maxim T. 
$scriptPath = Split-Path -parent $MyInvocation.MyCommand.Definition; 
. ($scriptPath + "\psFunctions.ps1");     # Adding script with reusable functions
. ($scriptPath + "\psSetVariables.ps1");  # Adding script to assign values to common variables

$logFile        = $scriptPath + "\Log\" + $MyInvocation.MyCommand.Name.Replace(".ps1",".txt"); 
(Get-Date).ToString("HH:mm:ss") + " --- Starting script " + $MyInvocation.MyCommand.Name | Out-File $logFile -Encoding OEM; # starting logging to file.
$logSummary = (Get-Date).ToString("HH:mm:ss") + " Script: GoogleWeb Hist".PadRight(28);
$quoteIDFile = $dataRootFolder + "\QuotesIntraDay\GoogleIntraday.txt"; if (Test-Path $quoteIDFile) { Remove-Item $quoteIDFile;} # Removing intraday file before each load

$listStart = $config.IndexOf("<GoogleWeb>"); $listEnd = $config.IndexOf("</GoogleWeb>"); 
if ($listStart -eq -1 -or $listEnd -eq -1 -or $listStart+1 -ge $listEnd) {"<GoogleWeb> Symbol list is empty. Exiting script." | Out-File $logFile -Encoding OEM -Append; exit(1);}
$symbolList = @($config | Select-Object -Index(($listStart+1)..($listEnd-1))); #list of symbols we will work on
"Symbol count: " + $symbolList.count + ". MinDate: $minDate" | Out-File $logFile -Encoding OEM -Append;

$urlBase   = "https://finance.google.com/finance/historical?q=@@Symbol@@&num=@@quotesPerPage@@&start=@@StartRow@@&startdate=@@StartDate@@" #https://www.google.com/finance/historical?q=MUTF_CA:TDB976&num=30&start=30&startdate=Jan+1%2C+1990
$mths = @("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
ForEach($sLine in $symbolList) { # For each symbol
    $ret = GetSymbolInfo $sLine $quotesFolder $minDate; $symbol = $ret[0]; $nextDate = $ret[1]; $symbolMaxDate = $ret[2]; $symbolMaxDateAdj = $ret[3]; $symbolQuoteFile = $ret[4]; $lastQuote = $ret[5];
    "Symbol: $symbol. Next date: $nextDate. Quote file: $symbolQuoteFile" | Out-File $logFile -Encoding OEM -Append;
    if ($nextDate -gt $symbolMaxDate) { "  We already have data up to maximum configuration date of $symbolMaxDate. Will not request new data for this symbol." | Out-File $logFile -Encoding OEM -Append; continue; } 

    $year = $nextDate.Substring(0,4); $month = [int]$nextDate.Substring(5,2) - 1; $day = $nextDate.Substring(8,2); $mthNotShifted = [int] $month+1; # Spliting date into parts. Month is 0 based
    $nextDateGO = $mths[$month]+"+"+[int] $day+"%2C+"+$year; # #$nextDate = '2012-10-01'; $nextDateGO = "Oct+1%2C+2012"
    $symbolGOW = $symbol.Replace("&","%26").Replace(":","%3A");
    
    $reqRows=0; # Total Quote count for that symbol
    $qpp=30; #default quotes per page value is 30;
    if ((New-TimeSpan $nextDate).Days -ge 30) {$qpp=200;} #Max quotes per page will be 200
    $urlSymbol = $urlBase.Replace("@@Symbol@@",$symbolGOW).Replace("@@quotesPerPage@@",$qpp).Replace("@@StartDate@@",$nextDateGO)
    $pg=0; # starting with page 0. 
    WHILE ($pg -le 50) { #expecting up to a 50 pages. Max days: 50x200=10,000 days = 27Years
        $quoteCount=0; $webpage = ""; $reqCount++;
        $url = $urlSymbol.Replace("@@StartRow@@",($pg*$qpp).ToString());
        "  Requesting url: " + $url | Out-File $logFile -Encoding OEM -Append;
        $wc = new-object system.net.WebClient;
        try {$webpage = $wc.DownloadData($url); } # Get page from url. This page will contain all quotes for single Symbol
        catch { $reqFailed++; "  " + $symbol + " - Not Found (web err) `r`n" | Out-File $logFile -Encoding OEM -Append; $pg=100; continue;} # if attempt to get webpage failed go to next symbol
        $str = [System.Text.Encoding]::ASCII.GetString($webpage); # This variable now contains downloaded quotes
    
        $startStr = "<table class=`"gf-table historical_price`">"; $endStr = "</table>";
        if (!$str.Contains($startStr)) {
            if ($reqRows -eq 0) { $reqFailed++; "  ** For Symbol: " + $symbol + " new quotes not found (there is no <table class=... in html that came back)`r`n" | Out-File $logFile -Encoding OEM -Append; } # if that symbol has no quotes, then record errors
            $pg=100; #Exiting page loop
            continue; # go to next Symbol
        }
        $str = $str.Substring($str.IndexOf($startStr) + $startStr.Length);
        $mfFlag = $true; if ($str.Contains("<th class=`"rgt bb`">Open")) {$mfFlag=$false;}; #page could have "Date,Close" values for Mutual Funds and "Date,Open,High,Low,Close,Volume" for stocks. Finding if this MF or Stock

        $reqSucceed++;
        $d2 = '2100-01-01'; #some future date
        $startStr1 = "<td class=`"lm`">"; $startStr2 = "<td class=`"rgt`">"; $startStr3 = "<td class=`"rgt rm`">"
        while($str.Contains($startStr1) -and $d2 -gt $nextDate) { #quote table row found
            $quoteCount++; 
            $str = $str.Substring($str.IndexOf($startStr1) + $StartStr1.Length); 
            $d = $str.Substring(0, $str.IndexOf("`n")); #Date

            #$d2 = [datetime]::ParseExact($d, "MMM d, yyyy", $null).ToString("yyyy-MM-dd"); # does not work when run on foreign language PC. Google returns Eng month name, PC translates to Dutch/Idian/German/...
            $mthName = $d.Substring(0,3); $qMonth= "0"+($mths.IndexOf($mthName) +1); $qMonth = $qMonth.Substring($qMonth.length-2,2); # converting month name into number: Feb->02
            $qYear = $d.Substring(7).Trim();
            $qDay = "0"+$d.Substring(4,2).Replace(",",""); $qDay = $qDay.Substring($qDay.Length-2, 2);
            $d2 = $qYear + "-" + $qMonth +"-" + $qDay

            if (!$mfFlag) { # just if this not mutual fund and page has Open,High,Low price columns, we need to skip them
                if (!$str.contains($startStr2)) {continue;} $str = $str.Substring($str.IndexOf($startStr2) + $StartStr2.Length); # Open Price - ignoring this column by removing value from our string
                if (!$str.contains($startStr2)) {continue;} $str = $str.Substring($str.IndexOf($startStr2) + $StartStr2.Length); # High Price - ignoring this column by removing value from our string
                if (!$str.contains($startStr2)) {continue;} $str = $str.Substring($str.IndexOf($startStr2) + $StartStr2.Length); # Low Price  - ignoring this column by removing value from our string
            }

            # Close Price - taking data from this column
            $cpStartStr = $startStr2; if ($mfFlag) {$cpStartStr = $startStr3;} # for close price start string is different for stocks and mutual fund
            if (!$str.contains($cpStartStr)) {continue;} #Price not found, continue to next record
            $str = $str.Substring($str.IndexOf($cpStartStr) + $cpStartStr.Length); 
            $cp = $str.Substring(0, $str.IndexOf("`n")).Replace(",","").Trim(); #Close Price
            if ($cp -eq "" -or $cp -eq "-") {continue;} #Value is empty or dash

            #if (!$str.contains($startStr3)) {continue;} $str = $str.Substring($str.IndexOf($startStr3) + $StartStr3.Length); # Volume - ignoring this column by removing value from our string

            if ($d2 -ge $nextDate -and $d2 -le $symbolMaxDateAdj) {
                $d2 + "," + $cp + "," + $symbol | Out-file $symbolQuoteFile -append -Encoding OEM; $reqRows++;
                if ($d2 -gt $lastQuote) {$lastQuote=$d2;} # Finding latest quote date from new records
            }
        } # while($str.Contains($startStr1)
        if ($quoteCount -lt $qpp) {break;}
        $pg++;
    } #WHILE ($pg -le 1000) {

    if ($reqRows -eq 0) {"Done: $symbol. No new quotes found. `r`n" | Out-File $logFile -Encoding OEM -Append;} # No new records where loaded
    else {"Done: $symbol (from: $nextDate). Record count: $reqRows. Last quote: $lastQuote `r`n" | Out-File $logFile -Encoding OEM -Append; $reqRowsT+=$reqRows;}

}
$duration = (NEW-TIMESPAN -Start $startTime -End (Get-Date)).TotalSeconds.ToString("#,##0") + " sec.";
(Get-Date).ToString("HH:mm:ss") + " --- Finished. Quotes Requested/Succeed/Failed/Rows: $reqCount/$reqSucceed/$reqFailed/$reqRowsT. Duration: $duration`r`n" | Out-File $logFile -Encoding OEM -append;
$logSummary + ". Quotes Requested/Succeed/Failed/Rows: $reqCount/$reqSucceed/$reqFailed/$reqRowsT. Duration: $duration";


